{-|
Module      : Test.Core.Text.DecarativeNgrams
Description :
Copyright   : (c) CNRS, 2017-Present
License     : AGPL + CECILL v3
Maintainer  : team@gargantext.org
Stability   : experimental
Portability : POSIX

-}

module Test.Core.Text.DeclarativeNgrams where

import Gargantext.Core (Lang(..))
import Gargantext.Core.Text.DeclarativeNgrams
import Gargantext.Prelude
import Test.Hspec

-- | Core.Text.DeclarativeNgrams tests
test :: Spec
test = do
  describe "test text matching" $ do
    it "exact match works" $ do
      let ngrams = Ngrams { terms = "hello", declaration = Exact }
      findNgrams [ngrams] "hello world, hello" `shouldBe`
        [ NgramsWithPos { ngrams, pos = 0, match = "hello" }
        , NgramsWithPos { ngrams, pos = 13, match = "hello" } ]
      findNgrams [ngrams] "hello world, Hello" `shouldBe`
        [ NgramsWithPos { ngrams, pos = 0, match = "hello" } ]
      findNgrams [ngrams] "hello world, hello, hello" `shouldBe`
        [ NgramsWithPos { ngrams, pos = 0, match = "hello" }
        , NgramsWithPos { ngrams, pos = 13, match = "hello" }
        , NgramsWithPos { ngrams, pos = 20, match = "hello" } ]
    it "case insensitive works" $ do
      let ngrams = Ngrams { terms = "Hello", declaration = CaseInsensitive }
      findNgrams [ngrams] "hello world, hello" `shouldBe`
        [ NgramsWithPos { ngrams, pos = 0, match = "hello" }
        , NgramsWithPos { ngrams, pos = 13, match = "hello" } ]
      findNgrams [ngrams] "hello world, Hello" `shouldBe`
        [ NgramsWithPos { ngrams, pos = 0, match = "hello" }
        , NgramsWithPos { ngrams, pos = 13, match = "Hello" } ]
      findNgrams [ngrams] "hello world, Hello, hEllo" `shouldBe`
        [ NgramsWithPos { ngrams, pos = 0, match = "hello" }
        , NgramsWithPos { ngrams, pos = 13, match = "Hello" }
        , NgramsWithPos { ngrams, pos = 20, match = "hEllo" } ]
    it "singular/plural works" $ do
      let ngrams = Ngrams { terms = "world", declaration = SingularPlural EN }
      findNgrams [ngrams] "hello world, hello" `shouldBe`
        [ NgramsWithPos { ngrams, pos = 6, match = "world" } ]
      findNgrams [ngrams] "hello worlds, hello" `shouldBe`
        [ NgramsWithPos { ngrams, pos = 6, match = "worlds" } ]
      findNgrams [ngrams] "hello world, Hello worlds" `shouldBe`
        [ NgramsWithPos { ngrams, pos = 6, match = "world" }
        , NgramsWithPos { ngrams, pos = 19, match = "worlds" } ]
      findNgrams [ngrams] "hello world, Hello, World" `shouldBe`
        [ NgramsWithPos { ngrams, pos = 6, match = "world" } ]
      findNgrams [ngrams] "hello worlds, Hello, world" `shouldBe`
        [ NgramsWithPos { ngrams, pos = 6, match = "worlds" }
        , NgramsWithPos { ngrams, pos = 21, match = "world" }]
         
